<?php

/**
 * This trait enables a model to be updated and to provide the changes
 */
namespace Intuit\Traits;

use Illuminate\Database\Eloquent\Concerns\HasAttributes;

trait CompareAttributes
{
	use HasAttributes;

	/* OVERRIDES */

	/* Since we use Jenson model, with HasAttributes, we need to add and override some basic dependancies */

	public function getIncrementing() { return FALSE; }
	public function usesTimestamps() { return FALSE; }
	public function getAttribute($key) {
		return (array_key_exists($key,$this->attributes)) ? $this->attributes[$key] : NULL;
	}

	/**
	 * Only record a change if it has changed
	 *
	 * @param $key
	 * @param $value
	 * @return void
	 */
	public function __set($key,$value) {
		if (! array_key_exists($key,$this->attributes) || (! $this->isSame($key,$value)))
			parent::__set($key,$value);
	}

	/**
	 * Check to see if a value has changed
	 *
	 * @param string $key
	 * @param $tgt
	 * @return bool
	 */
	private function isSame(string $key,$tgt): bool
	{
		$src = $this->getAttribute($key);

		return (is_object($src) || is_array($tgt)) ? count(array_diff_assoc((array)$src,(array)$tgt)) === 0 : $src === $tgt;
	}
}